@@ -24,7 +24,6 @@
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
-#include <libgen.h>
 #include <getopt.h>
 #include <dirent.h>
 #include <stdbool.h>
@@ -149,7 +148,7 @@ static int create_one_subvolume(const char *dst, struct btrfs_qgroup_inherit *in
 	int	fddst = -1;
 	char	*dupname = NULL;
 	char	*dupdir = NULL;
-	char	*newname;
+	const char *newname;
 	char	*dstdir;
 
 	ret = path_is_dir(dst);
@@ -170,7 +169,7 @@ static int create_one_subvolume(const char *dst, struct btrfs_qgroup_inherit *in
 		ret = -ENOMEM;
 		goto out;
 	}
-	newname = basename(dupname);
+	newname = path_basename(dupname);
 
 	dupdir = strdup(dst);
 	if (!dupdir) {
@@ -178,7 +177,7 @@ static int create_one_subvolume(const char *dst, struct btrfs_qgroup_inherit *in
 		ret = -ENOMEM;
 		goto out;
 	}
-	dstdir = dirname(dupdir);
+	dstdir = path_dirname(dupdir);
 
 	if (!test_issubvolname(newname)) {
 		error("invalid subvolume name: %s", newname);
@@ -349,8 +348,6 @@ static const char * const cmd_subvolume_delete_usage[] = {
 	OPTLINE("-c|--commit-after", "wait for transaction commit at the end of the operation"),
 	OPTLINE("-C|--commit-each", "wait for transaction commit after deleting each subvolume"),
 	OPTLINE("-i|--subvolid", "subvolume id of the to be removed subvolume"),
-	OPTLINE("--delete-qgroup", "also delete the qgroup 0/subvolid if it exists"),
-	OPTLINE("--no-delete-qgroup", "do not delete the qgroup 0/subvolid if it exists (default)"),
 	OPTLINE("-v|--verbose", "deprecated, alias for global -v option"),
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
@@ -364,7 +361,8 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 	int res, ret = 0;
 	int cnt;
 	int fd = -1;
-	char	*dname, *vname, *cpath;
+	char	*dname, *cpath;
+	const char *vname;
 	char	*dupdname = NULL;
 	char	*dupvname = NULL;
 	char	*path = NULL;
@@ -378,20 +376,15 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 	enum { COMMIT_AFTER = 1, COMMIT_EACH = 2 };
 	enum btrfs_util_error err;
 	uint64_t default_subvol_id, target_subvol_id = 0;
-	bool opt_delete_qgroup = false;
 
 	optind = 0;
 	while (1) {
 		int c;
-		enum { GETOPT_VAL_DELETE_QGROUP = GETOPT_VAL_FIRST,
-		       GETOPT_VAL_NO_DELETE_QGROUP };
 		static const struct option long_options[] = {
 			{"commit-after", no_argument, NULL, 'c'},
 			{"commit-each", no_argument, NULL, 'C'},
 			{"subvolid", required_argument, NULL, 'i'},
 			{"verbose", no_argument, NULL, 'v'},
-			{"delete-qgroup", no_argument, NULL, GETOPT_VAL_DELETE_QGROUP },
-			{"no-delete-qgroup", no_argument, NULL, GETOPT_VAL_NO_DELETE_QGROUP },
 			{NULL, 0, NULL, 0}
 		};
 
@@ -412,12 +405,6 @@ static int cmd_subvolume_delete(const struct cmd_struct *cmd, int argc, char **a
 		case 'v':
 			bconf_be_verbose();
 			break;
-		case GETOPT_VAL_DELETE_QGROUP:
-			opt_delete_qgroup = true;
-			break;
-		case GETOPT_VAL_NO_DELETE_QGROUP:
-			opt_delete_qgroup = false;
-			break;
 		default:
 			usage_unknown_option(cmd, argv);
 		}
@@ -482,9 +469,9 @@ again:
 		goto out;
 	}
 	dupdname = strdup(cpath);
-	dname = dirname(dupdname);
+	dname = path_dirname(dupdname);
 	dupvname = strdup(cpath);
-	vname = basename(dupvname);
+	vname = path_basename(dupvname);
 	free(cpath);
 
 	/* When subvolid is passed, <path> will point to the mount point */
@@ -553,19 +540,6 @@ again:
 			warning("deletion failed with EPERM, you don't have permissions or send may be in progress");
 		ret = 1;
 		goto out;
-	} else if (opt_delete_qgroup) {
-		struct btrfs_ioctl_qgroup_create_args args = { .qgroupid = target_subvol_id };
-
-		ret = ioctl(fd, BTRFS_IOC_QGROUP_CREATE, &args);
-		if (ret == 0) {
-			pr_verbose(LOG_DEFAULT, "Delete qgroup 0/%" PRIu64 "\n", target_subvol_id);
-		} else if (ret < 0 && (errno == ENOTCONN || errno == ENOENT)) {
-			/* Quotas not enabled, or there's no qgroup. */
-		} else {
-			warning("unable to delete qgroup 0/%llu: %m", subvolid);
-		}
-		/* Qgroup errors are not fatal. */
-		ret = 0;
 	}
 
 	if (commit_mode == COMMIT_EACH) {
@@ -670,7 +644,7 @@ static int cmd_subvolume_snapshot(const struct cmd_struct *cmd, int argc, char *
 	bool readonly = false;
 	char	*dupname = NULL;
 	char	*dupdir = NULL;
-	char	*newname;
+	const char *newname;
 	char	*dstdir;
 	enum btrfs_util_error err;
 	struct btrfs_ioctl_vol_args_v2	args;
@@ -727,13 +701,13 @@ static int cmd_subvolume_snapshot(const struct cmd_struct *cmd, int argc, char *
 
 	if (res > 0) {
 		dupname = strdup(subvol);
-		newname = basename(dupname);
+		newname = path_basename(dupname);
 		dstdir = dst;
 	} else {
 		dupname = strdup(dst);
-		newname = basename(dupname);
+		newname = path_basename(dupname);
 		dupdir = strdup(dst);
-		dstdir = dirname(dupdir);
+		dstdir = path_dirname(dupdir);
 	}
 
 	if (!test_issubvolname(newname)) {
@@ -972,9 +946,9 @@ static u64 find_root_gen(int fd)
 	 * them.
 	 */
 	sk->min_objectid = ino_args.treeid;
+	sk->min_type = BTRFS_ROOT_ITEM_KEY;
 	sk->max_objectid = ino_args.treeid;
 	sk->max_type = BTRFS_ROOT_ITEM_KEY;
-	sk->min_type = BTRFS_ROOT_ITEM_KEY;
 	sk->max_offset = (u64)-1;
 	sk->max_transid = (u64)-1;
 	sk->nr_items = 4096;
@@ -1138,6 +1112,10 @@ static char *ino_resolve(int fd, u64 ino, u64 *cache_dirid, char **cache_name)
 
 		name = (char *)(ref + 1);
 		name = strndup(name, namelen);
+		if (!name) {
+			error_msg(ERROR_MSG_MEMORY, NULL);
+			return NULL;
+		}
 
 		/* use our cached value */
 		if (dirid == *cache_dirid && *cache_name) {
@@ -1553,7 +1531,7 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 	struct btrfs_util_subvolume_iterator *iter;
 	struct btrfs_util_subvolume_info subvol;
 	char *subvol_path = NULL;
-	char *subvol_name = NULL;
+	const char *subvol_name = NULL;
 	enum btrfs_util_error err;
 	struct btrfs_qgroup_stats stats;
 	unsigned int unit_mode;
@@ -1665,7 +1643,7 @@ static int cmd_subvolume_show(const struct cmd_struct *cmd, int argc, char **arg
 		subvol_path = strdup("/");
 		subvol_name = "<FS_TREE>";
 	} else {
-		subvol_name = basename(subvol_path);
+		subvol_name = path_basename(subvol_path);
 	}
 
 	if (bconf.output_format == CMD_FORMAT_JSON) {
